import collections.abc

def nested_dict_get_values(key, dictionary):
    if hasattr(dictionary, 'items'):
        for k, v in dictionary.items():
            if k == key:
                yield v
            if isinstance(v, dict):
                for result in nested_dict_get_values(key, v):
                    yield result
            elif isinstance(v, list):
                for d in v:
                    for result in nested_dict_get_values(key, d):
                        yield result


def nested_dict_has_key(key, dictionary):

    if hasattr(dictionary, 'items'):
        for k, v in dictionary.items():
            if k == key:
                return True
            if isinstance(v, dict):
                if nested_dict_has_key(key, v):
                    return True
            elif isinstance(v, list):
                for d in v:
                    if nested_dict_has_key(key, d):
                        return True



def nested_dict_get_key_path(dict_obj, key, path=[]):
    """
    Description
    -----------
    Use this function to get the full path of a key in a dictionary.
    the dictionay can be a simple dictionary or a nested dictionary that may also contain lists of other nested dictionnaries.

    Parameters
    ----------
    @param dict_obj: the dictionary that can con contain the key
    @param key: the target key
    @return: a list containg the path sequence that leads to the key

    Example
    -------

    >>>from kdmt.dict import nested_dict_get_key_path
    >>> dictionary ={
        'k1': 'v1',
        'k2': 'v2',
        'k3':[{
            'k31': 'new_v31',
            'k32': 'v32'
        },
        {
            'k41': 'new_v31',
            'k42': 'v32'
        }
        ]
    }

    >>> print(nested_dict_get_key_path(dictionary, 'k31')
    #['k3', 0, 1]
    """
    for k, v in dict_obj.items():
        # add key to path
        path.append(k)
        if isinstance(v, dict):
            # continue searching
            nested_dict_get_key_path(v, key, path)
        elif isinstance(v, list):
            # search through list of dictionaries
            for i, item in enumerate(v):
                # add the index of list that item dict is part of, to path
                path.append(i)
                if isinstance(item, dict):
                    # continue searching in item dict
                    nested_dict_get_key_path(item, key, path)
                # if reached here, the last added index was incorrect, so removed
                path.pop()
        elif k == key:
            # add path to our result
            return path
        else:
            path=path[:-1]
    return path




def nested_dict_get_value(key, dictionary):

    if hasattr(dictionary, 'items'):
        for k, v in dictionary.items():
            if k == key:
                return v
            if isinstance(v, dict):
                val= nested_dict_get_value(key, v)
                if val:
                     return val
            elif isinstance(v, list):
                for d in v:
                    val = nested_dict_get_value(key, d)
                    if val:
                        return val

def update(origin, new):
    for k, v in new.items():
        if isinstance(v, collections.abc.Mapping):
            origin[k] = update(origin.get(k, {}), v)
        else:
            origin[k] = v
    return origin


def get_keys_for_value(dictionary, value):
    return [key for key,val in dictionary.items() if val == value]

if __name__=="__main__":
    dic={
        'k1': 'v1',
        'k2': 'v2',
        'k3':{
            'k31': 'v31'
        }
    }
    new ={
        'k1': 'v1',
        'k2': 'v2',
        'k3':[{
            'k31': 'new_v31',
            'k32': 'v32'
        },
        {
            'k41': 'new_v31',
            'k42': 'v32'
        }
        ]
    }
    r=nested_dict_get_key_path(new, 'k31' )
    print(r)